@@ -1,5 +1,5 @@
 # emacs: -*- mode: python; py-indent-offset: 4; tab-width: 4; indent-tabs-mode: nil -*-
-# ex: set sts=4 ts=4 sw=4 noet:
+# ex: set sts=4 ts=4 sw=4 et:
 # ## ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ### ##
 #
 #   See COPYING file distributed along with the datalad package for the
@@ -552,7 +552,7 @@ def test_GitRepo_fetch(orig_path, clone_path):
 
     # create a remote without an URL:
     origin.add_remote('not-available', 'git://example.com/not/existing')
-    origin.config.unset('remote.not-available.url', where='local')
+    origin.config.unset('remote.not-available.url', scope='local')
 
     # fetch without provided URL
     assert_raises(CommandError, origin.fetch, 'not-available')
@@ -582,7 +582,7 @@ def test_GitRepo_ssh_fetch(remote_path, repo_path):
 
     url = _path2localsshurl(remote_path)
     socket_path = op.join(str(ssh_manager.socket_dir),
-                          get_connection_hash('datalad-test', bundled=True))
+                          get_connection_hash('datalad-test'))
     repo = GitRepo(repo_path, create=True)
     repo.add_remote("ssh-remote", url)
 
@@ -613,7 +613,7 @@ def test_GitRepo_ssh_push(repo_path, remote_path):
     remote_repo = GitRepo(remote_path, create=True)
     url = _path2localsshurl(remote_path)
     socket_path = op.join(str(ssh_manager.socket_dir),
-                          get_connection_hash('datalad-test', bundled=True))
+                          get_connection_hash('datalad-test'))
     repo = GitRepo(repo_path, create=True)
     repo.add_remote("ssh-remote", url)
 
@@ -845,16 +845,16 @@ def test_GitRepo_dirty(path):
     ok_(repo.dirty)
 
     # User configuration doesn't affect .dirty's answer.
-    repo.config.set("diff.ignoreSubmodules", "all", where="local")
+    repo.config.set("diff.ignoreSubmodules", "all", scope="local")
     ok_(repo.dirty)
     # GitRepo.commit currently can't handle this setting, so remove it for the
     # save() calls below.
-    repo.config.unset("diff.ignoreSubmodules", where="local")
+    repo.config.unset("diff.ignoreSubmodules", scope="local")
     subm.save()
     repo.save()
     ok_(not repo.dirty)
 
-    repo.config.set("status.showUntrackedFiles", "no", where="local")
+    repo.config.set("status.showUntrackedFiles", "no", scope="local")
     create_tree(repo.path, {"untracked_dir": {"a": "a"}})
     ok_(repo.dirty)
 
@@ -944,141 +944,17 @@ def test_get_tracking_branch(o_path, c_path):
         clone.get_tracking_branch(remote_only=True))
 
 
-@with_tempfile
-@with_tempfile
-@with_tempfile
-def test_submodule_deinit(src, subsrc, path):
-    src = GitRepo(src)
-    subsrc = GitRepo(subsrc)
-    for repo in (src, subsrc):
-        for filename in ('some1.txt', 'some2.dat'):
-            with open(op.join(repo.path, filename), 'w') as f:
-                f.write(filename)
-            repo.add(filename)
-        repo.commit('Some files')
-    src.add_submodule('subm 1', name='subm 1', url=subsrc.path)
-    src.add_submodule('2', name='2', url=subsrc.path)
-    src.commit('submodule added')
-
-    top_repo = GitRepo.clone(src.path, path)
-    eq_({'subm 1', '2'},
-        {s["gitmodule_name"] for s in top_repo.get_submodules_()})
-    # note: here init=True is ok, since we are using it just for testing
-    with swallow_logs(new_level=logging.WARN) as cml:
-        top_repo.update_submodule('subm 1', init=True)
-        assert_in('Do not use update_submodule with init=True', cml.out)
-    top_repo.update_submodule('2', init=True)
-
-    # ok_(all([s.module_exists() for s in top_repo.get_submodules()]))
-    # TODO: old assertion above if non-bare? (can't use "direct mode" in test_gitrepo)
-    # Alternatively: New testrepo (plain git submodules) and have a dedicated
-    # test for annexes in addition
-    ok_(all(GitRepo.is_valid_repo(s["path"])
-            for s in top_repo.get_submodules_()))
-
-    # modify submodule:
-    with open(op.join(top_repo.path, 'subm 1', 'file_ut.dat'), "w") as f:
-        f.write("some content")
-
-    assert_raises(CommandError, top_repo.deinit_submodule, 'sub1')
-
-    # using force should work:
-    top_repo.deinit_submodule('subm 1', force=True)
-
-    ok_(not GitRepo.is_valid_repo(str(top_repo.pathobj / 'subm 1')))
-
-
-@with_tempfile(mkdir=True)
-@with_tempfile(mkdir=True)
-def test_GitRepo_add_submodule(source_path, path):
-    source = GitRepo(source_path, create=True)
-    with open(op.join(source_path, 'some.txt'), 'w') as f:
-        f.write("New text file.")
-    source.add('some.txt')
-    source.commit('somefile')
-
-    top_repo = GitRepo(path, create=True)
-
-    top_repo.add_submodule('sub', name='sub', url=source_path)
-    top_repo.commit('submodule added')
-    eq_([s["gitmodule_name"] for s in top_repo.get_submodules_()],
-        ['sub'])
-    assert_repo_status(path)
-    assert_repo_status(op.join(path, 'sub'))
-
-
-def test_GitRepo_update_submodule():
-    raise SkipTest("TODO")
-
-
-@with_tempfile(mkdir=True)
-def check_update_submodule_init_adjust_branch(is_ancestor, path):
-    src = GitRepo(op.join(path, "src"), create=True)
-    src_sub = GitRepo(op.join(src.path, "sub"), create=True)
-    src_sub.commit(msg="c0", options=["--allow-empty"])
-    src_sub.commit(msg="c1", options=["--allow-empty"])
-    src.add_submodule('sub', name='sub')
-    src.commit(msg="Add submodule")
-
-    # Move subdataset past the registered commit...
-    hexsha_registered = src_sub.get_hexsha()
-    if is_ancestor:
-        # ... where the registered commit is an ancestor of the new one.
-        src_sub.commit(msg="c2", options=["--allow-empty"])
-    else:
-        # ... where the registered commit is NOT an ancestor of the new one.
-        src_sub.call_git(["reset", "--hard", DEFAULT_BRANCH + "~1"])  # c0
-    hexsha_sub = src_sub.get_hexsha()
-
-    clone = GitRepo.clone(url=src.path,
-                          path=op.join(path, "clone"),
-                          create=True)
-    clone_sub = GitRepo.clone(url=src_sub.path,
-                              path=op.join(clone.path, "sub"),
-                              create=True)
-    ok_(clone.dirty)
-    eq_(clone_sub.get_active_branch(), DEFAULT_BRANCH)
-    eq_(hexsha_sub, clone_sub.get_hexsha())
-
-    clone.update_submodule("sub", init=True)
-
-    assert_false(clone.dirty)
-    eq_(hexsha_registered, clone_sub.get_hexsha())
-    if is_ancestor:
-        eq_(clone_sub.get_active_branch(), DEFAULT_BRANCH)
-    else:
-        assert_false(clone_sub.get_active_branch())
-
-
-def test_GitRepo_update_submodule_init_adjust_branch():
-    yield check_update_submodule_init_adjust_branch, True
-    yield check_update_submodule_init_adjust_branch, False
-
-
-@with_tempfile
-def test_update_submodules_sub_on_unborn_branch(path):
-    repo = GitRepo(path, create=True)
-    repo.commit(msg="c0", options=["--allow-empty"])
-    subrepo = GitRepo(op.join(path, "sub"), create=True)
-    subrepo.commit(msg="s c0", options=["--allow-empty"])
-    repo.add_submodule(path="sub")
-    subrepo.checkout("other", options=["--orphan"])
-    with assert_raises(ValueError) as cme:
-        repo.update_submodule(path="sub")
-    assert_in("unborn branch", str(cme.exception))
-
-
 @with_tempfile
 def test_GitRepo_get_submodules(path):
     repo = GitRepo(path, create=True)
 
     s_abc = GitRepo(op.join(path, "s_abc"), create=True)
     s_abc.commit(msg="c s_abc", options=["--allow-empty"])
-    repo.add_submodule(path="s_abc")
+    repo.save(path="s_abc")
 
     s_xyz = GitRepo(op.join(path, "s_xyz"), create=True)
     s_xyz.commit(msg="c s_xyz", options=["--allow-empty"])
-    repo.add_submodule(path="s_xyz")
+    repo.save(path="s_xyz")
 
     eq_([s["gitmodule_name"]
          for s in repo.get_submodules(sorted_=True)],
@@ -1090,7 +966,7 @@ def test_get_submodules_parent_on_unborn_branch(path):
     repo = GitRepo(path, create=True)
     subrepo = GitRepo(op.join(path, "sub"), create=True)
     subrepo.commit(msg="s", options=["--allow-empty"])
-    repo.add_submodule(path="sub")
+    repo.save(path="sub")
     eq_([s["gitmodule_name"] for s in repo.get_submodules_()],
         ["sub"])
 
@@ -1308,10 +1184,6 @@ def test_GitRepo_gitignore(path):
         gr.add('ignore.me')
     eq_(cme.exception.paths, ['ignore.me'])
 
-    with assert_raises(GitIgnoreError) as cme:
-        gr.add_submodule(path='ignore-sub.me')
-    eq_(cme.exception.paths, ['ignore-sub.me'])
-
     with assert_raises(GitIgnoreError) as cme:
         gr.add(['ignore.me', 'dontigno.re', op.join('ignore-sub.me', 'a_file.txt')])
     eq_(set(cme.exception.paths), {'ignore.me', 'ignore-sub.me'})
@@ -1341,7 +1213,7 @@ def test_GitRepo_set_remote_url(path):
     gr.add_remote('some-without-url', url2)
     eq_(gr.config['remote.some-without-url.url'], url2)
     # "remove" it
-    gr.config.unset('remote.some-without-url.url', where='local')
+    gr.config.unset('remote.some-without-url.url', scope='local')
     with assert_raises(KeyError):
         gr.config['remote.some-without-url.url']
     eq_(set(gr.get_remotes()), {'some', 'some-without-url'})
@@ -1646,9 +1518,7 @@ def test_gitrepo_call_git_methods(path):
     eq_(list(gr.call_git_items_(["ls-files"], read_only=True)),
         ["bar", "foo.txt"])
     eq_(list(gr.call_git_items_(["ls-files", "-z"], sep="\0", read_only=True)),
-        # Note: The custom separator has trailing empty item, but this is an
-        # arbitrary command with unknown output it isn't safe to trim it.
-        ["bar", "foo.txt", ""])
+        ["bar", "foo.txt"])
 
     with assert_raises(AssertionError):
         gr.call_git_oneline(["ls-files"], read_only=True)
@@ -1707,7 +1577,7 @@ def test_gitrepo_push_default_first_kludge(path):
 
     # But if datalad-push-default-first is set...
     cfg_var = f"remote.{DEFAULT_REMOTE}.datalad-push-default-first"
-    repo_b.config.set(cfg_var, "true", where="local")
+    repo_b.config.set(cfg_var, "true", scope="local")
     with swallow_logs(new_level=logging.DEBUG) as cml:
         res_twoshot = repo_b.push(remote=DEFAULT_REMOTE,
                                   refspec=[DEFAULT_BRANCH + ":b-twoshot",
